import datetime
import os

import arcpy
import pandas as pd

# Author J.Connolly USEPA

# Description: Generates acres table for species using the zone found in the species raster based union composites
# rather than the vector file input

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.


# Notes-
#       1)  Files with high edge effect may result in different acres values using the different format, using this
#           table rather than the one for the vector file will keep the calculation consistent
#       2) Note- if vector files were used for overlap ArcGIS would do a temporary conversion to raster resulting in
#           same difference seen with the composites.


# ASSUMPTIONS
# Species group is found in index position 1 of all input result tables when split by '_'
# All raster are 30 meter cells - note previously VI and CNMI has some uses with a different cell size
# regions is found in position 0 when split on the folder name

# ############################## User input
# folder with species files; this can be range or critical habitat; ESRI GRIDS
in_raster_location = r"path\Grids_byProjection"
# Lookup tables for the species zone files
look_up_fc = r"path\Lookup_R_Clipped_Union_CntyInter_HUC2ABInter_20190812"
look_up_value = 'HUCID'  # the key value for the species zone files often HUCID or ZoneID
outpath = r'out_path\Acres_Pixels'  # output location
# ###############user input variables
master_list = r"path\MasterListESA_Dec2018_June2020.csv"

# columns from the master species list that will be included in the final output
col_include_output = ['EntityID', 'Common Name', 'Scientific Name', 'Status', 'pop_abbrev', 'family', 'Lead Agency',
                      'Group', 'Des_CH', 'CH_GIS', 'Source of Call final BE-Range', 'WoE Summary Group',
                      'Source of Call final BE-Critical Habitat', 'Critical_Habitat_YesNo', 'Migratory',
                      'Migratory_YesNo', 'CH_Filename', 'Range_Filename', 'L48/NL48']

# Static variables
list_folder = os.listdir(in_raster_location)  # list of folder in the in location
today = datetime.datetime.today()  # time/day
date = today.strftime('%Y%m%d')  # date in YYYY/MM/DD
nl48_regions = ['AK', 'AS', 'CNMI', 'GU', 'HI', 'PR', 'VI', 'MP']  # list of nl48 regions
list_fc = os.listdir(look_up_fc)
find_file_type = in_raster_location.split(os.sep)

if 'Range' in find_file_type:
    file_type = "R_"
else:
    file_type = "CH_"

species_df = pd.read_csv(master_list, dtype=object)
[species_df.drop(m, axis=1, inplace=True) for m in species_df.columns.values.tolist() if m.startswith('Unnamed')]
base_sp_df = species_df.loc[:, col_include_output]
base_sp_df['EntityID'] = base_sp_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
if not os.path.exists(outpath):
    os.mkdir(outpath)


# ###Functions


def create_directory(dbf_dir):
    if not os.path.exists(dbf_dir):
        os.mkdir(dbf_dir)  # creates new folder


def parse_tables(in_table, in_row_sp):
    # set data type
    in_row_sp[look_up_value] = in_row_sp[look_up_value].map(lambda x: str(x).split('.')[0]).astype(str)
    # joins raster attribute table with the lookup table with the species entityIS
    merged_df = pd.merge(in_table, in_row_sp, on=look_up_value, how='right')

    sum_sp = merged_df[['COUNT', 'EntityID']].copy()
    # groups by entity id and sums
    sum_sp = sum_sp.groupby('EntityID').sum().reset_index()
    # selects the species that occur in the current regions - these will have a pixel count value >= 0
    sum_sp_region = sum_sp[sum_sp['COUNT'] >= 0]

    return sum_sp_region


start_time = datetime.datetime.now()
print "Start Time: " + start_time.ctime()

out_df = pd.DataFrame()
for folder in list_folder:  # loop over folder with grids
    arcpy.env.workspace = in_raster_location + os.sep + folder  # sets workspace
    out_location = outpath + os.sep + folder  # sets out
    list_raster = arcpy.ListRasters()  # list of raster in folder
    create_directory(out_location)  # creates output location if it doesn't exist
    region = folder.split("_")[0]  # extract region from folder name
    for raster in list_raster:  # loop over all rasters in folder
        # check for an existing working file, and if it is present load that files
        if os.path.exists(out_location + os.sep + raster + "_" + date + '.csv'):
            use_array = pd.read_csv(out_location + os.sep + raster + "_" + date + '.csv')
        else:
            # create raster layer
            my_raster_layer = arcpy.Raster(in_raster_location + os.sep + folder + os.sep + raster)
            if not my_raster_layer.hasRAT:
                print ("Building attribute table: {0}".format(in_raster_location + os.sep + folder + os.sep + raster))
                arcpy.BuildRasterAttributeTable_management(my_raster_layer, "Overwrite")
            # pulls in the species grid attribute tables to get the pixel counts
            raster_array = arcpy.da.TableToNumPyArray(in_raster_location + os.sep + folder + os.sep +
                                                      raster, ['VALUE', 'COUNT'])
            raster_zone_df = pd.DataFrame(data=raster_array, dtype=object)
            raster_zone_df[look_up_value] = raster_zone_df['VALUE'].map(lambda x: str(x).split('.')[0]).astype(str)
            # pulls in the species look-up table to link the back to the grid attribute table
            sp_zone_fc = [j for j in list_fc if j.startswith(file_type + raster.split("_")[1].title())]
            sp_zone_df = pd.read_csv(look_up_fc + os.sep + sp_zone_fc[0])
            if region == 'CONUS':
                sp_zone_df = sp_zone_df[~sp_zone_df['STUSPS'].isin(nl48_regions)]
            elif region == 'CNMI':  # CNMI is sometime marked as 'MP' for the STUSPS
                sp_zone_df = sp_zone_df[(sp_zone_df['STUSPS'] == region) | (sp_zone_df['STUSPS'] == 'MP')]
            else:
                sp_zone_df = sp_zone_df[sp_zone_df['STUSPS'] == region]
            print("Working on: {0} -  region {2}; raster {1}".format(sp_zone_fc[0], raster, folder))
            # parse the raster attribute table and the species lookup
            use_array = parse_tables(raster_zone_df, sp_zone_df)
            # converts the pixel counts to msq and then to acres
            use_array['MSQ'] = use_array['COUNT'].map(lambda x: int(x) * 900)
            use_array['Acres_' + region] = use_array['MSQ'].map(lambda x: int(x) * 0.000247)
            use_array.to_csv(out_location + os.sep + raster + "_" + date + '.csv')  # save working file

        if len(use_array) > 0:
            # set entity id data type
            use_array['EntityID'] = use_array['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)
            out_col = use_array[['EntityID', 'Acres_' + region]].copy()
            out_col.to_csv(out_location + os.sep + raster + '.csv')  # save working file
            if len(out_df) == 0:
                out_df = out_col.copy()
            else:
                # merge will cause multiple acres_region columns to occur;  by default suffix  eg _x
                out_df = pd.merge(out_df, out_col, on='EntityID', how='outer')

# merges final acres df with the species information from the master species list then save
common_col = [v for v in base_sp_df.columns.values.tolist() if v in out_df.columns.values.tolist()]
out_df = pd.merge(base_sp_df, out_df, on=common_col, how='left')
out_df.to_csv(outpath + os.sep + file_type + 'Temp_Acres_Pixels_' + date + '.csv')
clean_df = out_df.copy()  # copy df to clean up the acres columns (address the _x _y etc from the merge)

acres_headers = [v for v in clean_df.columns.values.tolist() if 'Acres' in v.split("_")]  # acres headers
non_acres = [v for v in clean_df.columns.values.tolist() if v not in acres_headers]  # non acres headers
acres_df = clean_df.ix[:, acres_headers]  # columns with acres values
non_acres_df = clean_df.ix[:, non_acres]  # columns without acres values (species info)

# transform dataframe to grp by acres_region
transform = acres_df.T.reset_index()
# splits the acres columns that were appended with x or y etc with merges to just Acres_region
transform['acres'] = transform['index'].apply(lambda s: s.split("_")[0] + "_" + s.split("_")[1])
transform.insert(0, 'acres_final', transform['acres'], allow_duplicates=True)
transform.drop(['index', 'acres'], axis=1, inplace=True)  # drops extra columns
# group-by an sum by acres_region then transform so acres are columns
sum_df = transform.groupby(['acres_final']).sum().T
acres_col = sum_df.columns.values.tolist()  # List acres cols
nl48_col = [v for v in acres_col if not v.endswith('CONUS')]  # list nl48 acres cols
concat_df = pd.concat([non_acres_df, sum_df], axis=1)  # concat back to the species df

concat_df['TotalAcresOnLand'] = concat_df[acres_col].sum(axis=1)  # calculate the total areas
concat_df['TotalAcresNL48'] = concat_df[nl48_col].sum(axis=1)  # calculate the total area in the nl48
concat_df.to_csv(outpath + os.sep + file_type + 'Acres_Pixels_' + date + '.csv')  # export

end = datetime.datetime.now()
print "End Time: " + end.ctime()
elapsed = end - start_time
print "Elapsed  Time: " + str(elapsed)
